/* 
 * PPRacer 
 * Copyright (C) 2004 Volker Stroebel <mmv1@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#include "audioconfig.h"

#include "game_config.h"
#include "fonts.h"

#include "ppgltk/ui_mgr.h"


bps_t bps[] = { 	{"8",0},
					{"16",1}};
						
freq_t freqs[] = { 	{"11025",0},
					{"22050",1},
					{"44100",2}};


AudioConfig::AudioConfig()
{
	title="Audio Configuration";
	pp::Vec2d pos(0,0);
	
	audioBox = new pp::CheckBox(pos, pp::Vec2d(32, 32) );
	audioBox->setState( getparam_no_audio());
	
	soundBox = new pp::CheckBox(pos, pp::Vec2d(32, 32) );
	soundBox->setState( getparam_sound_enabled());
		
	musicBox = new pp::CheckBox(pos, pp::Vec2d(32, 32) );
	musicBox->setState( getparam_music_enabled());	
	
	stereoBox = new pp::CheckBox(pos, pp::Vec2d(32, 32) );
	stereoBox->setState( getparam_audio_stereo());

	//frequency
	std::list<freq_t>::iterator freqit;
	bool found=false;
	
	for (unsigned int i=0; i<sizeof(freqs)/sizeof(freq_t); i++) {
		freqList.push_back(freqs[i]);	
		if (freqs[i].data==getparam_audio_freq_mode()){
			freqit = --freqList.end();
			found=true;
		}
    }
	if(!found) freqit = freqList.begin();
	
	freqListbox = new pp::Listbox<freq_t>( pos,
				   pp::Vec2d(150, 32),
				   "listbox_item",
				   freqList);
    freqListbox->setCurrentItem( freqit );
	
	//bits per sample
	std::list<freq_t>::iterator bpsit;
	found=false;
	
	for (unsigned int i=0; i<sizeof(bps)/sizeof(bps_t); i++) {
		bpsList.push_back(bps[i]);	
		if (bps[i].data==getparam_audio_format_mode()){
			bpsit = --bpsList.end();
			found=true;
		}
    }
	if(!found) bpsit = bpsList.begin();
	
	bpsListbox = new pp::Listbox<bps_t>( pos,
				   pp::Vec2d(150, 32),
				   "listbox_item",
				   bpsList);
    bpsListbox->setCurrentItem( bpsit );

}

AudioConfig::~AudioConfig()
{
	delete audioBox;
	delete soundBox;
	delete musicBox;
	delete stereoBox;
	delete bpsListbox;
	delete freqListbox;
}

void
AudioConfig::setWidgetPositions()
{
	int width = 500;
	int height = 240;

	pp::Vec2d pos(getparam_x_resolution()/2 - width/2,
				  getparam_y_resolution()/2 + height/2);
	
	Font font("button_label");
	
	font.draw("Sound Effects:",pos);
	soundBox->setPosition(pp::Vec2d(pos.x+width-32,pos.y));
	
	pos.y-=40;
	font.draw("Music:",pos);
	musicBox->setPosition(pp::Vec2d(pos.x+width-32,pos.y));
	
	pos.y-=80;
	font.draw("(needs restart)",pos);
	pos.y-=40;
	font.draw("Disable Audio:",pos);
	audioBox->setPosition(pp::Vec2d(pos.x+width-32,pos.y));
	
	pos.y-=40;
	font.draw("Stereo:",pos);
	stereoBox->setPosition(pp::Vec2d(pos.x+width-32,pos.y));
	
	pos.y-=40;
	font.draw("Bits Per Sample:",pos);
	bpsListbox->setPosition(pp::Vec2d(pos.x+width-150,pos.y));
	
	pos.y-=40;
	font.draw("Samples Per Second:",pos);
	freqListbox->setPosition(pp::Vec2d(pos.x+width-150,pos.y));
}

void
AudioConfig::apply()
{
	setparam_no_audio(bool( audioBox->getState() ));
	setparam_sound_enabled(bool( soundBox->getState() ));
	setparam_music_enabled(bool( musicBox->getState() ));
	setparam_audio_stereo(bool( stereoBox->getState() ));
	
	std::list<bps_t>::iterator bpsit = bpsListbox->getCurrentItem();
	setparam_audio_format_mode((*bpsit).data);
		
	std::list<freq_t>::iterator freqit = freqListbox->getCurrentItem();
	setparam_audio_freq_mode((*freqit).data);
		
	write_config_file();
	set_game_mode( GameMode::prevmode );
    UIMgr.setDirty();	
}
