/* 
 * PPRacer 
 * Copyright (C) 2004 Volker Stroebel <mmv1@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 
#include "configmode.h"

#include "ppgltk/button.h"
#include "ppgltk/ui_mgr.h"

#include "gl_util.h"
#include "fonts.h"
#include "render_util.h"
#include "game_config.h"

ConfigMode::ConfigMode()
{
	title="";
		
	pp::Vec2d pos(0,0);
	
	cancelBtn = new pp::Button(pos,
				     pp::Vec2d(300, 40),
				     "button_label",
				     "Cancel" );
    cancelBtn->setHilitFontBinding( "button_label_hilit" );
    cancelBtn->signalClicked.Connect(pp::CreateSlot(this,&ConfigMode::cancel));
   
	applyBtn = new pp::Button(pos,
				     pp::Vec2d(300, 40), 
				     "button_label", 
				     "Ok" );
    applyBtn->setHilitFontBinding( "button_label_hilit" );
    applyBtn->signalClicked.Connect(pp::CreateSlot(this,&ConfigMode::apply));
}

ConfigMode::~ConfigMode()
{
	delete cancelBtn;
	delete applyBtn;
}

void
ConfigMode::loop(double timeStep)
{
	update_audio();
    set_gl_options( GUI );
    clear_rendering_context();
    UIMgr.setupDisplay();
	drawTextandWidgets();
	setWidgetPositions();
    if (getparam_ui_snow()) {
		update_ui_snow( timeStep, false );
		draw_ui_snow();
    }
    theme.drawMenuDecorations();
    UIMgr.draw();
    reshape( getparam_x_resolution(), getparam_y_resolution() );
    winsys_swap_buffers();
}

void
ConfigMode::drawTextandWidgets()
{
	int w = getparam_x_resolution();
    int h = getparam_y_resolution();
	
	Font font("heading",true,true);	
	font.draw(title,pp::Vec2d(w/2,h/2+200));

	cancelBtn->setPosition(pp::Vec2d(w/2-300,h/2-240));
	applyBtn->setPosition(pp::Vec2d(w/2,h/2-240));
}

bool
ConfigMode::keyPressEvent(SDLKey key)
{
	switch (key){
		case SDLK_ESCAPE:
			cancel();
	    	return true;
		case SDLK_RETURN:
			apply();
	    	return true;
		default:
			return false;
	}
}

void
ConfigMode::cancel()
{
	set_game_mode( GameMode::prevmode );
    UIMgr.setDirty();
}
