/* 
 * PPRacer 
 * Copyright (C) 2003-2004 The PPRacer Team
 *
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "event_race_select.h"

#include "game_config.h"

#include "ppgltk/ui_mgr.h"
#include "ppgltk/ui_theme.h"
#include "ppgltk/button.h"
#include "ppgltk/ssbutton.h"
#include "ppgltk/listbox.h"
#include "ppgltk/ui_snow.h"
#include "ppgltk/textarea.h"

#include "ppgltk/audio/audio.h"

#include "loop.h"
#include "render_util.h"
#include "gl_util.h"
#include "course_load.h"
#include "fonts.h"
#include "textures.h"
#include "joystick.h"

#include "stuff.h"
#include "game_mgr.h"
#include "player.h"

#include <iostream>


//static std::list<CourseData> *courseList;
static std::list<CourseData>::iterator curElem;

//static EventData *event_data;
//static CupData *cup_data;

EventRaceSelect::EventRaceSelect()
{
    pp::Vec2d pos(0,0);

	//event_data = gameMgr->getCurrentEvent();
	//cup_data = gameMgr->getCurrentCup();
	//courseList = &cup_data->raceList;

    back_btn = new pp::Button(pos,
			      pp::Vec2d(150, 40), 
			      "button_label", 
			      "Back" );
    back_btn->setHilitFontBinding( "button_label_hilit" );
    back_btn->signalClicked.Connect(pp::CreateSlot(this,&EventRaceSelect::back));

    start_btn = new pp::Button(pos,
			       pp::Vec2d(150, 40),
			       "button_label",
			       "Race!" );
    start_btn->setHilitFontBinding( "button_label_hilit" );
    start_btn->setDisabledFontBinding( "button_label_disabled" );
	start_btn->signalClicked.Connect(pp::CreateSlot(this,&EventRaceSelect::start));

	if ( GameMode::prevmode != GAME_OVER ) {
		curElem = gameMgr->getCurrentCup().raceList.begin();
    } else {
		if( gameMgr->wasRaceWon() ){
			if(curElem != --gameMgr->getCurrentCup().raceList.end()){
				curElem++;
			}
		}
    }

	race_listbox = new pp::Listbox<CourseData>(pos,
				   pp::Vec2d(460, 44),
				   "listbox_item",
				   gameMgr->getCurrentCup().raceList);
    race_listbox->setCurrentItem( curElem );
	race_listbox->signalChange.Connect(pp::CreateSlot(this,&EventRaceSelect::listboxItemChange));
    // Create text area 
     
    desc_ta = new pp::Textarea( pos,
			       pp::Vec2d(312, 107),
			       "race_description",
			       "" );
	desc_ta->setText( (*curElem).description );
    // Create state buttons - only if practicing or if cup_complete
    
  	conditions_ssbtn = NULL;
	wind_ssbtn = NULL;
	snow_ssbtn = NULL;
	mirror_ssbtn = NULL;
    
    updateStates();

    play_music( "start_screen" );
}

EventRaceSelect::~EventRaceSelect()
{
	delete back_btn;
	delete start_btn;
	delete race_listbox;
	delete conditions_ssbtn;
	delete snow_ssbtn;
	delete wind_ssbtn;
	delete mirror_ssbtn;
    delete desc_ta;		
}

void
EventRaceSelect::loop(double timeStep)
{
	update_audio();

    set_gl_options( GUI );

    clear_rendering_context();

    UIMgr.setupDisplay();

    if (getparam_ui_snow()) {
		update_ui_snow( timeStep, 
			bool( wind_ssbtn != NULL && 
				  wind_ssbtn->getState() ) );
		draw_ui_snow();
    }

    theme.drawMenuDecorations();

    setWidgetPositionsAndDrawDecorations();

    UIMgr.draw();

    reshape( getparam_x_resolution(), getparam_y_resolution() );

    winsys_swap_buffers();
}

void
EventRaceSelect::setWidgetPositionsAndDrawDecorations()
{
    int w = getparam_x_resolution();
    int h = getparam_y_resolution();
    int box_width, box_height, box_max_y;
    int x_org, y_org;
    char *current_course;
    GLuint texobj;

    // set the dimensions of the box in which all widgets should fit
    box_width = 460;
    box_height = 310;
    box_max_y = h - 128;

    x_org = w/2 - box_width/2;
    y_org = h/2 - box_height/2;
    if ( y_org + box_height > box_max_y ) {
	y_org = box_max_y - box_height;
    }

    back_btn->setPosition(pp::Vec2d( x_org + 131 - back_btn->getWidth()/2.0,
		      42 ) );

    start_btn->setPosition(pp::Vec2d( x_org + 343 - start_btn->getWidth()/2.0,
		      42 ) );

	race_listbox->setPosition(pp::Vec2d( x_org,y_org + 221 ) );

	desc_ta->setPosition(pp::Vec2d( x_org,y_org + 66 ) );
    
	// Draw tux life icons
	int i;
	
	glPushMatrix();
	{

	    glTranslatef( x_org + box_width - 4*36 + 4,
			  y_org + 181,
			  0 );
	    
	    check_assertion( INIT_NUM_LIVES == 4, 
			     "Assumption about number of lives invalid -- "
			     "need to recode this part" );

	    if ( !get_texture_binding( "tux_life", &texobj ) ) {
		texobj = 0;
	    }

	    glBindTexture( GL_TEXTURE_2D, texobj );

	    for ( i=0; i<4; i++ ) {
		pp::Vec2d ll, ur;
		if ( players[0].getLives() > i ) {
		    ll = pp::Vec2d( 0, 0.5 );
		    ur = pp::Vec2d( 1, 1 );
		} else {
		    ll = pp::Vec2d( 0, 0 );
		    ur = pp::Vec2d( 1, 0.5 );
		}
		glBegin( GL_QUADS );
		{
		    glTexCoord2f( ll.x, ll.y );
		    glVertex2f( 0, 0 );

		    glTexCoord2f( ur.x, ll.y );
		    glVertex2f( 32, 0 );

		    glTexCoord2f( ur.x, ur.y );
		    glVertex2f( 32, 32 );

		    glTexCoord2f( ll.x, ur.y );
		    glVertex2f( 0, 32 );
		}
		glEnd();

		glTranslatef( 36, 0, 0 );
	    }
	}
	glPopMatrix();
    
    // Draw other stuff
	
	Font font("menu_label",true,true);
	font.draw("Select a race",pp::Vec2d(x_org + box_width/2.0,
											y_org + 310));
	
	// Draw text indicating race requirements (if race not completed), 
    //   or results in race if race completed.
    drawStatusMsg( x_org, y_org, box_width, box_height );

    // Draw preview
	std::list<CourseData>::iterator elem;
	elem = race_listbox->getCurrentItem();
	current_course = (*elem).course;

    glDisable( GL_TEXTURE_2D );

    glColor4f( 1.0, 1.0, 1.0, 1.0 );
    glBegin( GL_QUADS );
    {
		glVertex2f( x_org+box_width-140, y_org+66 );
		glVertex2f( x_org+box_width, y_org+66 );
		glVertex2f( x_org+box_width, y_org+66+107 );
		glVertex2f( x_org+box_width-140, y_org+66+107 );
    }
    glEnd();

    glEnable( GL_TEXTURE_2D );

    if ( !get_texture_binding( current_course, &texobj ) ) {
	if ( !get_texture_binding( "no_preview", &texobj ) ) {
	    texobj = 0;
	}
    }

    glBindTexture( GL_TEXTURE_2D, texobj );

    glBegin( GL_QUADS );
    {
	glTexCoord2d( 0, 0);
	glVertex2f( x_org+box_width-136, y_org+70 );

	glTexCoord2d( 1, 0);
	glVertex2f( x_org+box_width-4, y_org+70 );

	glTexCoord2d( 1, 1);
	glVertex2f( x_org+box_width-4, y_org+70+99 );

	glTexCoord2d( 0, 1);
	glVertex2f( x_org+box_width-136, y_org+70+99 );
    }
    glEnd();
}

void
EventRaceSelect::drawStatusMsg( int x_org, int y_org, int box_width, int box_height )
{
    char *msg;
    char buff[BUFF_LEN];
    bool draw_stats = true;	
	
	switch(state){
		case DEAD:
			msg = "You don't have any lives left.";
	    	draw_stats = false;
			break;
		
		case RACEWON:
			msg = "Race won! Your result:";	
			break;
		
		case PLAYABLE:
			msg = "Needed to advance:";
			break;
		
		case UNPLAYABLE:
			msg = "You can't enter this race yet.";
			draw_stats = false;
			break;
		default:
			msg = "";
			draw_stats = false;
			break;
	}
		
	Font font("race_requirements");
	Font labelfont("race_requirements_label");
	
	labelfont.draw(msg,pp::Vec2d(x_org,y_org + 200));
	
	if ( draw_stats ) {
		int minutes;
		int seconds;
		int hundredths;

		pp::Vec2d pos(x_org,y_org + 184);
		
		getTimeComponents( data.time, minutes, seconds, hundredths );

		labelfont.draw("Time:", pos);
		
		pos.x+=40;
		sprintf( buff, "%02d:%02d.%02d",minutes, seconds, hundredths);
		font.draw(buff, pos);
		
		pos.x+=80;
		labelfont.draw("Herring:", pos);
		
		pos.x+=55;
		sprintf( buff, "%03d", data.herring ); 
		font.draw(buff, pos);
		
		pos.x+=45;
		labelfont.draw("Score:", pos);

		pos.x+=42;
		sprintf( buff, "%06d", data.score );
		font.draw(buff, pos);
	}
		
	pp::Vec2d pos(x_org,y_org+50);
	labelfont.draw("Contributed by:",pos);
	pos.x+=106;	
	
	if((*curElem).contributed !=NULL){
		font.draw((*curElem).contributed,pos);
	}else{
		font.draw("Unknown",pos);
	}
}


void
EventRaceSelect::updateStates()
{
	if ( players[0].getLives() <= 0 ) {
		state=DEAD;
		start_btn->setEnabled( false );
	} else {
		data.won=false;
		players[0].getCupCourseData(
				std::string(gameMgr->getCurrentEvent().name),
				std::string(gameMgr->getCurrentCup().name),
				std::string((*curElem).name),
				data);
	
		if(data.won){
			state=RACEWON;
			start_btn->setEnabled( false );	
		}else{
			
			difficulty_level_t d = gameMgr->difficulty;
			data.time = (*curElem).time_req[d];
			data.herring = (*curElem).herring_req[d];
			data.score = int((*curElem).score_req[d]);

			if(curElem==gameMgr->getCurrentCup().raceList.begin()){
				state=PLAYABLE;
				start_btn->setEnabled( true );
			}else{
				PlayerCourseData prev;
				curElem--;
				players[0].getCupCourseData(
					std::string(gameMgr->getCurrentEvent().name),
					std::string(gameMgr->getCurrentCup().name),
					std::string((*curElem).name),
					prev);
				curElem++;
				
				if(prev.won){
					state=PLAYABLE;
					start_btn->setEnabled( true );
				} else {
					state=UNPLAYABLE;
					start_btn->setEnabled( false );
				}
			}
		}
	}
}

void
EventRaceSelect::back()
{
	set_game_mode( EVENT_SELECT );
    UIMgr.setDirty();
}

void
EventRaceSelect::start()
{
    start_btn->setHighlight( true );
    loop( 0 );
	gameMgr->setCurrentRace(curElem);

    set_game_mode( LOADING );
}


void
EventRaceSelect::listboxItemChange()
{
	curElem=race_listbox->getCurrentItem();
	desc_ta->setText( (*curElem).description );
	updateStates();
	UIMgr.setDirty();
}

bool
EventRaceSelect::keyPressEvent(SDLKey key)
{
	switch (key){
		case SDLK_UP:
		case SDLK_LEFT:
			race_listbox->gotoPrevItem();
	    	return true;
		case SDLK_RIGHT:
		case SDLK_DOWN:
			race_listbox->gotoNextItem();
	    	return true;
		case SDLK_RETURN:
	    	start_btn->simulateMouseClick();
			UIMgr.setDirty();
			return true;
		case SDLK_ESCAPE:
	    	back_btn->simulateMouseClick();
			UIMgr.setDirty();
			return true;
		case 'c': 
	    	conditions_ssbtn->simulateMouseClick();
	    	return true;
		case 'w': 
	    	wind_ssbtn->simulateMouseClick();
	    	return true;
		case 'm':
	    	mirror_ssbtn->simulateMouseClick();
	    	return true;
		default:
			return false;
	}	
}
