/* 
 * PPRacer 
 * Copyright (C) 2003-2004 The PPRacer Team
 *
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "event_select.h"

#include "ppgltk/ui_mgr.h"
#include "ppgltk/ui_theme.h"
#include "ppgltk/button.h"
#include "ppgltk/listbox.h"
#include "ppgltk/ui_snow.h"

#include "ppgltk/audio/audio.h"

#include "loop.h"
#include "render_util.h"
#include "gl_util.h"
#include "course_load.h"
#include "fonts.h"
#include "textures.h"
#include "joystick.h"

#include "game_config.h"

#include "player.h"
#include "game_mgr.h"
	
EventSelect::EventSelect()
{
	pp::Vec2d pos(0,0);
	
	if(GameMode::prevmode==EVENT_RACE_SELECT){
		curEvent = gameMgr->currentEvent;
		curCup = gameMgr->currentCup;
		if(players[0].isCupComplete((*curEvent).name,(*curCup).name)){
			if(curCup != --(*curEvent).cupList.end()){
				curCup++;
			}				
		}
	}else{
		curEvent = eventList.begin();
		curCup = (*curEvent).cupList.begin();
	}
	updateCupStates();	
		
	backBtn = new pp::Button( pos,
			      pp::Vec2d(150, 40), 
			      "button_label", 
			      "Back" );
    backBtn->setHilitFontBinding( "button_label_hilit" );
    backBtn->signalClicked.Connect(pp::CreateSlot(this,&EventSelect::back));

    continueBtn = new pp::Button( pos,
			       pp::Vec2d(150, 40),
			       "button_label",
			       "Continue" );
    continueBtn->setHilitFontBinding( "button_label_hilit" );
    continueBtn->setDisabledFontBinding( "button_label_disabled" );
    continueBtn->signalClicked.Connect(pp::CreateSlot(this,&EventSelect::apply));

	eventListbox = new pp::Listbox<EventData>( pos,
				   pp::Vec2d(400, 40),
				   "listbox_item",
				   eventList);
	eventListbox->setCurrentItem( curEvent );
	eventListbox->signalChange.Connect(pp::CreateSlot(this,&EventSelect::eventChanged));
	
	cupListbox = new pp::Listbox<CupData>( pos,
				   pp::Vec2d(400, 40),
				   "listbox_item",
				   (*curEvent).cupList );
	cupListbox->setCurrentItem( curCup );	
    cupListbox->signalChange.Connect(pp::CreateSlot(this,&EventSelect::cupChanged));

	updateButtonEnabledStates();
}

EventSelect::~EventSelect()
{
	delete backBtn;
	delete continueBtn;	
	delete eventListbox;
	delete cupListbox;
}

void
EventSelect::loop(double timeStep)
{
    update_audio();
    set_gl_options( GUI );
    clear_rendering_context();
    UIMgr.setupDisplay();
    if (getparam_ui_snow()) {
		update_ui_snow( timeStep, false );
		draw_ui_snow();
    }
    theme.drawMenuDecorations();
    setWidgetPositions();
    UIMgr.draw();
    reshape( getparam_x_resolution(), getparam_y_resolution() );
    winsys_swap_buffers();
}

void
EventSelect::setWidgetPositions()
{
    int width = getparam_x_resolution();
    int height = getparam_y_resolution();
    
	pp::Vec2d pos(width/2,height/2);
	
	backBtn->setPosition(pp::Vec2d(pos.x-250,pos.y-240));
	continueBtn->setPosition(pp::Vec2d(pos.x+50,pos.y-240));
	
	{
		pos.y+=100;
		Font font("menu_label",true,true);
		font.draw("Select event and cup",pos);
	}
	
	{
		pos.y-=60;
		Font font("event_and_cup_label");
		font.draw("Event:",pp::Vec2d(pos.x-200,pos.y));
	
		pos.y-=50;
		eventListbox->setPosition(pp::Vec2d(pos.x-200,pos.y));
	
		pos.y-=40;
		font.draw("Cup:",pp::Vec2d(pos.x-200,pos.y));
	
		pos.y-=50;
		cupListbox->setPosition(pp::Vec2d(pos.x-200,pos.y));
	}
	
	{
	    char* string;
		if(curCupComplete){
			string = "You've won this cup!";
		}else if (curCupPlayable){
			string = "You must complete this cup next";
		}else{
			string = "You cannot enter this cup yet"; 
		}
				
		pos.y-=20;
		Font font("cup_status",true,true);
		font.draw(string,pp::Vec2d(pos.x,pos.y));
	}
}

void
EventSelect::updateCupStates()
{
	curCupComplete=false;
	curCupPlayable=false;
	
	if ( players[0].isCupComplete( (*curEvent).name, (*curCup).name) ) {
		curCupComplete=true;
		curCupPlayable=true;
    } else if ( players[0].isFirstIncompleteCup( curEvent,curCup ) ) {
		curCupPlayable=true;
    }
}

void
EventSelect::updateButtonEnabledStates()
{
	if(curCupPlayable){
		continueBtn->setEnabled(true);
	}else{
		continueBtn->setEnabled(false);
	}
}

void
EventSelect::cupChanged()
{
	curCup = cupListbox->getCurrentItem();
	updateCupStates();
	updateButtonEnabledStates();
}

void
EventSelect::eventChanged()
{
	curEvent = eventListbox->getCurrentItem();
	curCup = (*curEvent).cupList.begin();
	updateCupStates();
	updateButtonEnabledStates();
}

void
EventSelect::back()
{
	set_game_mode( GAME_TYPE_SELECT );
	UIMgr.setDirty();
}

void
EventSelect::apply()
{
    curEvent = eventListbox->getCurrentItem();
	curCup = cupListbox->getCurrentItem();

	gameMgr->setupEventAndCup(	curEvent,
								curCup );

	if(!getparam_always_save_event_race_data()){	
	if(!players[0].isCupComplete((*curEvent).name, (*curCup).name )){
		players[0].clearCupData((*curEvent).name, (*curCup).name);
	}
	}
	players[0].resetLives();
    set_game_mode( EVENT_RACE_SELECT );
    UIMgr.setDirty();
}

bool
EventSelect::keyPressEvent(SDLKey key)
{
	switch (key){
		case SDLK_UP:
			cupListbox->gotoPrevItem();
	    	return true;
		case SDLK_DOWN:
			cupListbox->gotoNextItem();
	    	return true;
		case SDLK_LEFT:
			eventListbox->gotoPrevItem();
	    	return true;
		case SDLK_RIGHT:
			eventListbox->gotoPrevItem();
	    	return true;
		case SDLK_RETURN: 
	    	continueBtn->simulateMouseClick();
			UIMgr.setDirty();
			return true;
		case SDLK_ESCAPE: 
	    	backBtn->simulateMouseClick();
			UIMgr.setDirty();
			return true;
		default:
			return false;
	}	
}
