/* 
 * PPRacer 
 * Copyright (C) 2004 Volker Stroebel <mmv1@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 
#include "graphicsconfig.h"

#include "game_config.h"
#include "fonts.h"

#include "game_mgr.h"

#include "ppgltk/ui_mgr.h"

	
GraphicsConfig::GraphicsConfig()
{
	title="Graphics Configuration";	
	
	pp::Vec2d pos(0,0);
	
	uiSnowBox = new pp::CheckBox(pos, pp::Vec2d(32, 32) );
	uiSnowBox->setState( getparam_ui_snow());

	fpsBox = new pp::CheckBox(pos, pp::Vec2d(32, 32) );
	fpsBox->setState( getparam_display_fps());

	fogBox = new pp::CheckBox(pos, pp::Vec2d(32, 32) );
	fogBox->setState( !getparam_disable_fog());

	reflectionsBox = new pp::CheckBox(pos, pp::Vec2d(32, 32) );
	reflectionsBox->setState( getparam_terrain_envmap());

	shadowsBox = new pp::CheckBox(pos, pp::Vec2d(32, 32) );
	shadowsBox->setState( getparam_draw_tux_shadow());
}

GraphicsConfig::~GraphicsConfig()
{
	delete uiSnowBox;
	delete fpsBox;
	delete fogBox;
	delete reflectionsBox;
	delete shadowsBox;
}

void
GraphicsConfig::setWidgetPositions()
{
	int width = 500;
	int height = 240;
		
	pp::Vec2d pos(getparam_x_resolution()/2 - width/2,
				  getparam_y_resolution()/2 + height/2);
	
	Font font("button_label");
	
	font.draw("Show UI Snow:",pos);
	uiSnowBox->setPosition(pp::Vec2d(pos.x+width-32,pos.y));
	
	pos.y-=40;
	font.draw("Display FPS:",pos);
	fpsBox->setPosition(pp::Vec2d(pos.x+width-32,pos.y));
	
	pos.y-=40;
	font.draw("Draw Fog:",pos);
	fogBox->setPosition(pp::Vec2d(pos.x+width-32,pos.y));
	
	pos.y-=40;
	font.draw("Reflections:",pos);
	reflectionsBox->setPosition(pp::Vec2d(pos.x+width-32,pos.y));
	
	pos.y-=40;
	font.draw("Shadows:",pos);
	shadowsBox->setPosition(pp::Vec2d(pos.x+width-32,pos.y));
}

void
GraphicsConfig::apply()
{
	setparam_ui_snow(bool( uiSnowBox->getState() ));
	setparam_display_fps(bool( fpsBox->getState() ));
	setparam_disable_fog(!bool(fogBox->getState() ));
	setparam_terrain_envmap(bool( reflectionsBox->getState() ));
	setparam_draw_tux_shadow(bool( shadowsBox->getState() ));
	
	write_config_file();
	set_game_mode( GameMode::prevmode );
    UIMgr.setDirty();	
}
