/* 
 * PPRacer 
 * Copyright (C) 2003-2004 The PPRacer Team
 *
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "fonts.h"
#include "gl_util.h"
#include "textures.h"
#include "fps.h"
#include "phys_sim.h"
#include "course_mgr.h"
#include "game_config.h"
#include "hud.h"

#include "stuff.h"
#include "game_mgr.h"

#include "ppgltk/ui_mgr.h"
#include "ppgltk/alg/defs.h"

HUD HUD1;

HUD::Element::Element()
{
	type=-1;
	x=0;
	y=0;
	binding=NULL;
	width=0;
	height=0;
	size=0;
	string=NULL;
	angle=0;
}

HUD::HUD(void){
	reset();	
}

void
HUD::reset()
{
	num_elements=0;
}

bool
HUD::add(Element new_element){
	if(num_elements<HUD_MAX_ITEMS){
		element[num_elements]=new_element;
		num_elements++;
		return true;
	}else{
		return false;
	}	
}
	
bool
HUD::update(const int i, Element new_element){
	if(num_elements>i){
		element[i]=new_element;
		return true;	
	}else{
		//element i not available
		return false;
	}	
}



void
HUD::draw(Player& plyr)
{
	UIMgr.setupDisplay();
	
	for(int i=0; i<num_elements;i++){
		switch(element[i].type){
			case 0:
				text(i);
				break;
			case 1:
				fps(i);
				break;
			case 2:
				herring(i,plyr.herring);
				break;
			case 3:
				image(i);
				break;
			case 4:
				time(i);
				break;	
			case 5:
				{
				pp::Vec3d vel = plyr.vel;
    			speed(i,vel.normalize()* M_PER_SEC_TO_KM_PER_H);
				}
				break;
			case 6:
				{
				pp::Vec3d vel = plyr.vel;
				gauge(i,vel.normalize()* M_PER_SEC_TO_KM_PER_H,plyr.control.jump_amt);
				}
				break;
			case 7:
				bar(i,plyr.control.jump_amt);
				break;
			case 8:
				{
				pp::Vec3d vel = plyr.vel;
				bar(i,vel.normalize()* M_PER_SEC_TO_KM_PER_H/135);
				}	
				break;
		}
	}
	
	//draw_hud(plyr);
}


void
HUD::text(const int i)
{
    font_t font;
    int w, asc, desc;
	
    set_gl_options( TEXFONT );
    glColor3f( 1.0, 1.0, 1.0 );

    if ( !get_font_binding( element[i].binding, font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font for binding %s", element[i].binding );
	return;
    }

    get_font_metrics( &font, element[i].string, &w, &asc, &desc );


	bind_font_texture( &font );
	fix_xy(element[i].x,element[i].y,asc,w);
	
	glPushMatrix();
	{
		glTranslatef(element[i].x,element[i].y,0);
		draw_string( &font, element[i].string );
    }
	glPopMatrix();
}

void
HUD::fps(const int i)
{
	if ( ! getparam_display_fps() ) {
		return;
    }
	
    char string[BUFF_LEN];
    font_t font;
	
    if ( !get_font_binding( element[i].binding, font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font for binding %s", element[i].binding );
	return;
    }

    bind_font_texture( &font );
    set_gl_options( TEXFONT );
    glColor3f( 1, 1, 1 );

    sprintf( string, element[i].string, fpsCounter.get() );

	fix_xy(element[i].x,element[i].y);
	
    glPushMatrix();
    {
	glTranslatef( element[i].x,
		      element[i].y,
		      0 );
	draw_string( &font, string );
    }
    glPopMatrix();	
}

void
HUD::herring(const int i, const int herring_count)
{
    char string[BUFF_LEN];
	
    font_t font;
    int w, asc, desc;

    set_gl_options( TEXFONT );
    glColor3f( 1.0, 1.0, 1.0 );
  

    if ( !get_font_binding( element[i].binding, font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font for binding %s", element[i].binding);
	return;
    }

    sprintf( string, element[i].string, herring_count ); 

    get_font_metrics( &font, string, &w, &asc, &desc );


	bind_font_texture( &font );
	fix_xy(element[i].x,element[i].y,asc,w);
	glPushMatrix();
	{
		glTranslatef(element[i].x,element[i].y,0);
		draw_string( &font, string );
    }
	glPopMatrix();
}

void
HUD::image(const int i)
{
    GLuint texobj;
	
    set_gl_options( TEXFONT );
    glColor3f( 1.0, 1.0, 1.0 );

    if ( !get_texture_binding( element[i].binding, &texobj ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get texture for binding %s",element[i].binding );
	return;
    }

    glBindTexture( GL_TEXTURE_2D, texobj );

	fix_xy(element[i].x,element[i].y,element[i].height,element[i].width);
	
    glPushMatrix();
    {
	glTranslatef(element[i].x, element[i].y,0);

	glBegin( GL_QUADS );
	{
	   	glTexCoord2f( 0, 0 );
	    glVertex2f( 0, 0 );

	    glTexCoord2f( double(element[i].width) / element[i].size,
			  0 );
	    glVertex2f( element[i].width, 0 );

	    glTexCoord2f( 
			double(element[i].width) / element[i].size,
			double(element[i].height) / element[i].size );
	    glVertex2f( element[i].width, element[i].height );

	    glTexCoord2f( 
			0,
			double(element[i].height) / element[i].size );
	    glVertex2f( 0, element[i].height );
	}
	glEnd();
    }
    glPopMatrix();
}

void
HUD::time(const int i)
{
    char string[BUFF_LEN];
	int minutes, seconds, hundredths;
	
    font_t font;
    int w, asc, desc;
	
    getTimeComponents( gameMgr->time, minutes, seconds, hundredths );

    set_gl_options( TEXFONT );
    glColor3f( 1.0, 1.0, 1.0 );

    if ( !get_font_binding( element[i].binding, font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font for binding %s", element[i].binding );
	return;
    }

    sprintf( string, element[i].string, minutes, seconds, hundredths);

    get_font_metrics( &font, string, &w, &asc, &desc );


	bind_font_texture( &font );
	fix_xy(element[i].x,element[i].y,asc,w);
	glPushMatrix();
	{
		glTranslatef(element[i].x,element[i].y,0);
		draw_string( &font, string );
    }
	glPopMatrix();
}

void
HUD::speed(const int i, const double speed)
{
	font_t font;
    int w, asc, desc;
	char string[BUFF_LEN];

    set_gl_options( TEXFONT );
    glColor3f( 1.0, 1.0, 1.0 );

    if ( !get_font_binding( element[i].binding, font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font for binding %s", element[i].binding );
	return;
    }
    sprintf(string,element[i].string,speed);

    get_font_metrics( &font, string, &w, &asc, &desc );

	bind_font_texture( &font );
	fix_xy(element[i].x,element[i].y,asc);
		
	glPushMatrix();
	{
		glTranslatef(element[i].x-w/2.0,element[i].y,0);
		draw_string( &font, string );
    }
	glPopMatrix();	
}

void
HUD::bar(const int i, double percentage)
{
	GLuint texobj;
	
	if(percentage>1)percentage=1;
	
	double temp_sin=sin(double(element[i].angle)/180.0*M_PI);
	double temp_cos=cos(double(element[i].angle)/180.0*M_PI);
	
    set_gl_options( TEXFONT );

    if ( !get_texture_binding( element[i].binding, &texobj ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get texture for binding %s",element[i].binding );
	return;
    }

    glBindTexture( GL_TEXTURE_2D, texobj );

	fix_xy(element[i].x,element[i].y,int(element[i].height));
	
    glPushMatrix();
    {
	glTranslatef(element[i].x, element[i].y,0);

	glBegin( GL_QUADS );
	{
		glTexCoord2f(0,0);
	    glVertex2f(0,0);

	    glTexCoord2f(1,0);
	    glVertex2f(temp_cos*element[i].width,temp_sin*element[i].width);

    	glTexCoord2f(1,percentage);
    	glVertex2f(temp_cos*element[i].width+temp_sin*element[i].height*percentage,
			temp_sin*element[i].width-temp_cos*element[i].height*percentage);

    	glTexCoord2f(0,percentage);
    	glVertex2f(temp_sin*element[i].height*percentage, (-1)*temp_cos*element[i].height*percentage);

	}
	glEnd();
    }
    glPopMatrix();
	
}

//Sollte man bei Gelegenheit noch konfigurierbar machen
#define ENERGY_GAUGE_BOTTOM 3.0
#define ENERGY_GAUGE_CENTER_X 71.0
#define ENERGY_GAUGE_CENTER_Y 55.0
#define SPEEDBAR_OUTER_RADIUS ( ENERGY_GAUGE_CENTER_X )
#define SPEEDBAR_BASE_ANGLE 225
#define SPEEDBAR_MAX_ANGLE 45
#define SPEEDBAR_GREEN_MAX_SPEED ( MAX_PADDLING_SPEED * M_PER_SEC_TO_KM_PER_H )
#define SPEEDBAR_YELLOW_MAX_SPEED 100
#define SPEEDBAR_RED_MAX_SPEED 160
#define SPEEDBAR_GREEN_FRACTION 0.5
#define SPEEDBAR_YELLOW_FRACTION 0.25
#define SPEEDBAR_RED_FRACTION 0.25
#define SPEED_UNITS_Y_OFFSET 4.0

static GLfloat energy_background_color[] = { 0.2, 0.2, 0.2, 0.5 };
static GLfloat energy_foreground_color[] = { 0.54, 0.59, 1.00, 0.5 };
static GLfloat speedbar_background_color[] = { 0.2, 0.2, 0.2, 0.5 };
static GLfloat white[] = { 1.0, 1.0, 1.0, 1.0 };

void
HUD::gauge(const int i, const double speed, const double energy)
{
	char *binding;
    GLfloat xplane[4] = { 1.0/element[i].size, 0.0, 0.0, 0.0 };
    GLfloat yplane[4] = { 0.0, 1.0/element[i].size, 0.0, 0.0 };
    GLuint energymask_texobj, speedmask_texobj, outline_texobj;
    font_t speed_font;
    font_t units_font;
    int w, asc, desc;
    char *string;
    char buff[BUFF_LEN];
    double y;
    double speedbar_frac;

    set_gl_options( GAUGE_BARS );

    binding = "gauge_energy_mask";
    if ( !get_texture_binding( binding, &energymask_texobj ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get texture for binding %s", binding );
	return;
    }

    binding = "gauge_speed_mask";
    if ( !get_texture_binding( binding, &speedmask_texobj ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get texture for binding %s", binding );
	return;
    }

    binding = "gauge_outline";
    if ( !get_texture_binding( binding, &outline_texobj ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get texture for binding %s", binding );
	return;
    }

    binding = "speed_digits";
    if ( !get_font_binding( binding, speed_font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font for binding %s", &speed_font );
    }

    binding = "speed_units";
    if ( !get_font_binding( binding, units_font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font for binding %s", &speed_font );
    }


    glTexGenfv( GL_S, GL_OBJECT_PLANE, xplane );
    glTexGenfv( GL_T, GL_OBJECT_PLANE, yplane );

    glPushMatrix();
    {
	glTranslatef( getparam_x_resolution() - element[i].width,
		      0,
		      0 );

	glColor4fv( energy_background_color );

	glBindTexture( GL_TEXTURE_2D, energymask_texobj );

	y = ENERGY_GAUGE_BOTTOM + energy * element[i].height;

	glBegin( GL_QUADS );
	{
	    glVertex2f( 0.0, y );
	    glVertex2f( element[i].size, y );
	    glVertex2f( element[i].size, element[i].size );
	    glVertex2f( 0.0, element[i].size );
	}
	glEnd();

	glColor4fv( energy_foreground_color );

	glBegin( GL_QUADS );
	{
	    glVertex2f( 0.0, 0.0 );
	    glVertex2f( element[i].size, 0.0 );
	    glVertex2f( element[i].size, y );
	    glVertex2f( 0.0, y );
	}
	glEnd();

	/* Calculate the fraction of the speed bar to fill */
	speedbar_frac = 0.0;

	if ( speed > SPEEDBAR_GREEN_MAX_SPEED ) {
	    speedbar_frac = SPEEDBAR_GREEN_FRACTION;
	    
	    if ( speed > SPEEDBAR_YELLOW_MAX_SPEED ) {
		speedbar_frac += SPEEDBAR_YELLOW_FRACTION;
		
		if ( speed > SPEEDBAR_RED_MAX_SPEED ) {
		    speedbar_frac += SPEEDBAR_RED_FRACTION;
		} else {
		    speedbar_frac +=
			( speed - SPEEDBAR_YELLOW_MAX_SPEED ) /
			( SPEEDBAR_RED_MAX_SPEED - SPEEDBAR_YELLOW_MAX_SPEED ) *
			SPEEDBAR_RED_FRACTION;
		}

	    } else {
		speedbar_frac += 
		    ( speed - SPEEDBAR_GREEN_MAX_SPEED ) /
		    ( SPEEDBAR_YELLOW_MAX_SPEED - SPEEDBAR_GREEN_MAX_SPEED ) *
		    SPEEDBAR_YELLOW_FRACTION;
	    }
	    
	} else {
	    speedbar_frac +=  speed/SPEEDBAR_GREEN_MAX_SPEED * 
		SPEEDBAR_GREEN_FRACTION;
	}

	glColor4fv( speedbar_background_color );

	glBindTexture( GL_TEXTURE_2D, speedmask_texobj );

	draw_partial_tri_fan( 1.0 );

	glColor4fv( white );

	draw_partial_tri_fan( MIN( 1.0, speedbar_frac ) );

	glColor4fv( white );

	glBindTexture( GL_TEXTURE_2D, outline_texobj );

	glBegin( GL_QUADS );
	{
	    glVertex2f( 0.0, 0.0 );
	    glVertex2f( element[i].size, 0.0 );
	    glVertex2f( element[i].size, element[i].size );
	    glVertex2f( 0.0, element[i].size );
	}
	glEnd();



	sprintf( buff, "%d", (int)speed );
	string = buff;

	get_font_metrics( &speed_font, string, &w, &asc, &desc );

	bind_font_texture( &speed_font);
	set_gl_options( TEXFONT );
	glColor4f( 1, 1, 1, 1 );

	glPushMatrix();
	{
	    glTranslatef( ENERGY_GAUGE_CENTER_X - w/2.0,
			  ENERGY_GAUGE_BOTTOM + element[i].height / 2.0,
			  0 );
	    //draw_string( speed_font, string );
	    
	}
	glPopMatrix();

	string = "km/h";

	get_font_metrics( &units_font, string, &w, &asc, &desc );

	bind_font_texture( &units_font );

	glPushMatrix();
	{
	    glTranslatef( ENERGY_GAUGE_CENTER_X - w/2.0,
			  ENERGY_GAUGE_BOTTOM + element[i].height / 2.0
			  - asc - SPEED_UNITS_Y_OFFSET,
			  0 );
	    //draw_string( units_font, string );
	    
	}
	glPopMatrix();
	
    }
    glPopMatrix();
}

#define CIRCLE_DIVISIONS 10

void 
HUD::draw_partial_tri_fan(const double fraction)
{
    int divs;
    double angle, angle_incr, cur_angle;
    int i;
    bool trifan = false;
    pp::Vec2d pt;

    angle = SPEEDBAR_BASE_ANGLE + 
	( SPEEDBAR_MAX_ANGLE - SPEEDBAR_BASE_ANGLE ) * fraction;

    divs = int(( SPEEDBAR_BASE_ANGLE - angle ) * CIRCLE_DIVISIONS / 360.0);

    cur_angle = SPEEDBAR_BASE_ANGLE;

    angle_incr = 360.0 / CIRCLE_DIVISIONS;

    for (i=0; i<divs; i++) {
	if ( !trifan ) {
	    start_tri_fan();
	    trifan = true;
	}

	cur_angle -= angle_incr;

	pt = calc_new_fan_pt( cur_angle );

	glVertex2f( pt.x, pt.y );
    }

    if ( cur_angle > angle + EPS ) {
	cur_angle = angle;
	if ( !trifan ) {
	    start_tri_fan();
	    trifan = true;
	}

	pt = calc_new_fan_pt( cur_angle );

	glVertex2f( pt.x, pt.y );
    }

    if ( trifan ) {
	glEnd();
	trifan = false;
    }
}

pp::Vec2d
HUD::calc_new_fan_pt(const double angle )
{
    pp::Vec2d pt;
    pt.x = ENERGY_GAUGE_CENTER_X + cos( ANGLES_TO_RADIANS( angle ) ) *
	SPEEDBAR_OUTER_RADIUS;
    pt.y = ENERGY_GAUGE_CENTER_Y + sin( ANGLES_TO_RADIANS( angle ) ) *
	SPEEDBAR_OUTER_RADIUS;

    return pt;
}

void
HUD::start_tri_fan(void)
{
    pp::Vec2d pt;

    glBegin( GL_TRIANGLE_FAN );
    glVertex2f( ENERGY_GAUGE_CENTER_X, 
		ENERGY_GAUGE_CENTER_Y );

    pt = calc_new_fan_pt( SPEEDBAR_BASE_ANGLE ); 

    glVertex2f( pt.x, pt.y );
}

void
HUD::fix_xy(int &x, int &y, const int asc, const int w)
{
	if(x<0){
		x=getparam_x_resolution()+x-w;
	}
	if(y<0){
		y=getparam_y_resolution()+y-asc;
	}
}
