/* 
 * PPRacer 
 * Copyright (C) 2003-2004 The PPRacer Team
 * 
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "paused.h"



#include "ppgltk/ui_mgr.h"
#include "ppgltk/audio/audio.h"

#include "game_config.h"
#include "gl_util.h"
#include "fps.h"
#include "render_util.h"
#include "phys_sim.h"
#include "view.h"
#include "course_render.h"
#include "tux.h"
#include "tux_shadow.h"
#include "loop.h"
#include "fog.h"
#include "viewfrustum.h"
#include "hud.h"
#include "part_sys.h"
#include "fonts.h"
#include "joystick.h"
#include "course_mgr.h"

#include "game_mgr.h"



Paused::Paused()
{
	int centerX = getparam_x_resolution() / 2;
	int centerY = getparam_y_resolution() / 2;
	
	backgroundFrm = new pp::Frame(pp::Vec2d(centerX-120,centerY-100),
								  pp::Vec2d(240,200));

	configBtn = new pp::Button(pp::Vec2d(centerX-150,centerY-15),
				     	pp::Vec2d(300, 40),
				     	"button_label",
				     	"Configuration" );
	
    configBtn->setHilitFontBinding("button_label_hilit");
	configBtn->signalClicked.Connect(pp::CreateSlot(this,&Paused::configuration));
	
	resumeBtn = new pp::Button(pp::Vec2d(centerX-150,centerY-55),
				     	pp::Vec2d(300, 40),
				     	"button_label",
				     	"Resume" );
	
    resumeBtn->setHilitFontBinding("button_label_hilit");
	resumeBtn->signalClicked.Connect(pp::CreateSlot(this,&Paused::resume));

	quitBtn = new pp::Button(pp::Vec2d(centerX-150,centerY-95),
				     	pp::Vec2d(300, 40),
				     	"button_label",
				     	"Quit" );
	
    quitBtn->setHilitFontBinding("button_label_hilit");
	quitBtn->signalClicked.Connect(pp::CreateSlot(this,&Paused::quit));

	play_music( "paused" );
}

Paused::~Paused()
{
	delete configBtn;
	delete resumeBtn;
	delete quitBtn;
	delete backgroundFrm;
}
	
void
Paused::loop(double timeStep)
{
    int width, height;
    width = getparam_x_resolution();
    height = getparam_y_resolution();

    // Check joystick 
    if ( is_joystick_active() ) {
		update_joystick();

		if ( is_joystick_continue_button_down() )
		{
			set_game_mode( RACING );
			winsys_post_redisplay();
		    return;
		}
    }

    fpsCounter.update();

    update_audio();

    clear_rendering_context();

    fogPlane.setup();

    update_player_pos( players[0], 0 );
    update_view( players[0], 0 );

    setup_view_frustum( players[0], NEAR_CLIP_DIST, 
			getparam_forward_clip_distance() );

    draw_sky( players[0].view.pos );

    draw_fog_plane( );

    set_course_clipping( true );
    set_course_eye_point( players[0].view.pos );
    setup_course_lighting();
    render_course();
    draw_trees();

    if ( getparam_draw_particles() ) {
		draw_particles( players[0] );
    }

    draw_tux();
    draw_tux_shadow();

    set_gl_options( GUI );

    UIMgr.setupDisplay();

    HUD1.draw(players[0]);
	
    UIMgr.draw();
	drawText();
    reshape( width, height );

    winsys_swap_buffers();
}

void
Paused::drawText()
{
    int w = getparam_x_resolution();
    int h = getparam_y_resolution();
    int x_org, y_org;
    int box_width, box_height;
    char *string;
    int string_w, asc, desc;
    font_t font;

    box_width = 200;
    box_height = 300;

    x_org = int(w/2.0 - box_width/2.0);
    y_org = int(h/2.0 - box_height/2.0 + 60);   

    if ( !get_font_binding( "paused", font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font for binding paused" );
    } else {
	string = "Paused";

	get_font_metrics( &font, string, &string_w, &asc, &desc );
	
	glPushMatrix();
	{
	    glTranslatef( x_org + box_width/2.0 - string_w/2.0,
			  y_org + box_height/2.0, 
			  0 );
	    bind_font_texture( &font );
	    draw_string( &font, string );
	}
	glPopMatrix();
    }
}

bool
Paused::keyPressEvent(SDLKey key)
{
	set_game_mode( RACING );
	winsys_post_redisplay();
	return true;
}

void
Paused::resume()
{
	set_game_mode( RACING );
	winsys_post_redisplay();
}

void
Paused::quit()
{
	gameMgr->abortRace();
    set_game_mode( GAME_OVER );
}

void
Paused::configuration()
{
	set_game_mode( CONFIGURATION );
	UIMgr.setDirty();
}
