/* 
 * PPRacer 
 * Copyright (C) 2004 Peter Reichel <peter@apps4linux.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "signal.h"

namespace pp{


class BaseSlot;
class BaseSignal;
class BaseArgument;

class BaseArgument {
public:
	BaseArgument(int cnt)
	{
		mCnt = cnt;
	}
	virtual ~BaseArgument() { }
	int Count() const { return mCnt; }
private:
	int mCnt;
};

class Argument0 : public BaseArgument {
public:
	Argument0() : BaseArgument(0) { }
	~Argument0() { }
};

//Slotklasse

class BaseSlot {
public:
	BaseSlot() { }
	virtual ~BaseSlot() { }

	virtual void Call(BaseArgument *arg) = 0;
};


//Signalklasse

class BaseSignal {
public:
	BaseSignal()
	{
		mSlot = NULL;
	}
	~BaseSignal()
	{
		Clean();
	}
	
	void Connect(BaseSlot *slot)
	{
		Clean();
		mSlot = slot;
	}
private:
	void Clean()
	{
		if (mSlot) {
			delete mSlot;
			mSlot = NULL;
		}
	}
protected:
	BaseSlot *mSlot;
};

class Signal0 : public BaseSignal {
public:
	Signal0() { }
	~Signal0() { }

	void Emit()
	{
		if (mSlot) {
			Argument0 arg;
			mSlot->Call(&arg);
		}
	}
};

}// namepsace pp
