/* 
 * PPRacer 
 * Copyright (C) 2003-2004 The PPRacer Team
 *
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "ui_theme.h"
#include "ui_mgr.h"
#include "fonts.h"

#define DEFAULT_ARROW_BUTTON_HEIGHT 15
#define DEFAULT_ARROW_BUTTON_WIDTH  27
#define DEFAULT_ARROW_REGION_WIDTH  36
#define DEFAULT_ARROW_VERT_SEPARATION  4
#define DEFAULT_BORDER_WIDTH 4
#define LDEFAULT_TEXT_PAD 8


namespace pp{

template <class T>
Listbox<T>::Listbox(  pp::Vec2d pos, pp::Vec2d size, 
				char *font_binding, 
				list_type& item_list)
	: items(item_list)
{
    char *binding;
    pp::Vec2d ll;
    pp::Vec2d ur;

    this->position = pos;
    this->size = size;
	
    this->arrow_width = DEFAULT_ARROW_REGION_WIDTH;
    this->border_width = DEFAULT_BORDER_WIDTH;
    this->text_pad = LDEFAULT_TEXT_PAD;
    this->arrow_vert_separation = DEFAULT_ARROW_VERT_SEPARATION;
    this->font_binding = font_binding;
    this->border_colour = theme.foreground;
    this->background_colour = theme.background; 

    /* Create up arrow button */
    this->up_button = new pp::Button( 
		pp::Vec2d( 0, 0 ), /* position will be set later */
		pp::Vec2d(DEFAULT_ARROW_BUTTON_WIDTH,DEFAULT_ARROW_BUTTON_HEIGHT),
		NULL,
		NULL );

    binding = "listbox_arrows";

    ll = pp::Vec2d( 2.0/64.0, 17.0/64.0 );
    ur = pp::Vec2d( 29.0/64.0, 32.0/64.0 );
    this->up_button->setImage( binding, ll, ur, pp::Color::white );

    ll = pp::Vec2d( 34.0/64.0, 17.0/64.0 );
    ur = pp::Vec2d( 61.0/64.0, 32.0/64.0 );
    this->up_button->setDisabledImage( binding, ll, ur, pp::Color::white );

    ll = pp::Vec2d( 34.0/64.0, 49.0/64.0 );
    ur = pp::Vec2d( 61.0/64.0, 64.0/64.0 );
    this->up_button->setHilitImage( binding, ll, ur, pp::Color::white );

    ll = pp::Vec2d( 2.0/64.0, 49.0/64.0 );
    ur = pp::Vec2d( 29.0/64.0, 64.0/64.0 );
    this->up_button->setClickedImage( binding, ll, ur, pp::Color::white );

	this->up_button->setActive( true );
    this->up_button->signalClicked.Connect(pp::CreateSlot(this,&Listbox::upButtonClick));


    /* Create down arrow button */
    this->down_button = new pp::Button( 
		pp::Vec2d( 0, 0 ), /* position will be set later */
		pp::Vec2d(DEFAULT_ARROW_BUTTON_WIDTH,DEFAULT_ARROW_BUTTON_HEIGHT),
		NULL,
		NULL );

    binding = "listbox_arrows";

    ll = pp::Vec2d( 2.0/64.0, 1.0/64.0 );
    ur = pp::Vec2d( 29.0/64.0, 16.0/64.0 );
    this->down_button->setImage( binding, ll, ur, pp::Color::white );

    ll = pp::Vec2d( 34.0/64.0, 1.0/64.0 );
    ur = pp::Vec2d( 61.0/64.0, 16.0/64.0 );
    this->down_button->setDisabledImage( binding, ll, ur, pp::Color::white );

    ll = pp::Vec2d( 34.0/64.0, 33.0/64.0 );
    ur = pp::Vec2d( 61.0/64.0, 48.0/64.0 );
    this->down_button->setHilitImage(binding, ll, ur,pp::Color::white );

    ll = pp::Vec2d( 2.0/64.0, 33.0/64.0 );
    ur = pp::Vec2d( 29.0/64.0, 48.0/64.0 );
    this->down_button->setClickedImage( binding, ll, ur, pp::Color::white );

	this->down_button->setActive( true );
    this->down_button->signalClicked.Connect(pp::CreateSlot(this,&Listbox::downButtonClick));

    this->items = item_list;
    this->cur_item = this->items.begin();

    //this->label_gen_func = func;

    this->visible = false;
    this->active = false;

    updateButtonEnabledStates();
    updateButtonPositions();
}

template <class T>
Listbox<T>::~Listbox()
{
    setVisible( false );
    setActive( false );

    delete up_button;
    delete down_button;
}

template <class T>
void
Listbox<T>::updateButtonEnabledStates()
{
    if ( items.empty() ) {
		/* No items in list */
		up_button->setEnabled( false );
		down_button->setEnabled( false );
    } else {
		if ( cur_item == items.begin() ) {
			up_button->setEnabled( false );
		} else {
			up_button->setEnabled( true );
		}

		if ( cur_item == --items.end() ) {
			down_button->setEnabled( false );
		} else {
			down_button->setEnabled( true );
		}
    }
}

template <class T>
void
Listbox<T>::updateButtonPositions()
{
	up_button->setPosition(pp::Vec2d( 
	    position.x + size.x - up_button->getWidth(),
	    position.y + size.y / 2.0 + arrow_vert_separation / 2.0 ) );

    down_button->setPosition(pp::Vec2d( 
	    position.x + size.x - down_button->getWidth(),
	    position.y + size.y / 2.0 - 
	    arrow_vert_separation / 2.0 -
	    up_button->getHeight() ) );
}

template <class T>
void
Listbox<T>::setPosition(pp::Vec2d pos)
{
	Widget::setPosition(pos);
	updateButtonPositions();
}

template <class T>
void
Listbox<T>::setCurrentItem( list_iterator item )
{
    cur_item = item;
    updateButtonEnabledStates();
}

template <class T>	
void
Listbox<T>::setItemList( list_type& item_list )
{
    this->items = item_list;
    this->cur_item = items.begin();;
    updateButtonEnabledStates();

    UIMgr.setDirty();
}

template <class T>
bool 
Listbox<T>::gotoNextItem()
{
    if ( cur_item == --items.end() ) {
		return false;
    }
    down_button->simulateMouseClick();
    return true;
}

template <class T>
bool
Listbox<T>::gotoPrevItem()
{
    if ( cur_item == items.begin() ) {
		return false;
    }
    up_button->simulateMouseClick();
    return true;
}

template <class T>
void
Listbox<T>::upButtonClick()
{
	if(cur_item != items.begin()){
		cur_item--; // = get_prev_list_elem( item_list,cur_item );
		signalChange.Emit();
    	updateButtonEnabledStates();
    	UIMgr.setDirty();
	}
}

template <class T>
void
Listbox<T>::downButtonClick()
{
	if(cur_item != --items.end()){
		cur_item++;
		signalChange.Emit();
    	updateButtonEnabledStates();
    	UIMgr.setDirty();
	}	
}

template <class T>
void
Listbox<T>::draw()
{
    font_t font;

    glDisable( GL_TEXTURE_2D );

    glColor3dv( (double*)&border_colour );

    glRectf( position.x, 
	     position.y,
	     position.x + size.x - arrow_width,
	     position.y + size.y );

    glColor3dv( (double*)&background_colour );

    glRectf( position.x + border_width, 
	     position.y + border_width,
	     position.x + size.x - border_width - arrow_width,
	     position.y + size.y - border_width );

    glEnable( GL_TEXTURE_2D );

    if ( !get_font_binding( font_binding, font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font object for binding %s",
		       font_binding );
    } else {
	int w, asc, desc;

	get_font_metrics( &font, (*cur_item).name, &w, &asc, &desc );

	bind_font_texture( &font );

	glColor3f( 1.0, 1.0, 1.0 );

	glPushMatrix();
	{
	    glTranslatef( 
		position.x + border_width + text_pad,
		position.y + size.y/2.0 - asc/2.0 + desc/2.0,
		0 );

	    draw_string( &font, (*cur_item).name);
	}
	glPopMatrix();
    }

    up_button->draw();
    down_button->draw();
		
}
	

} //namespace pp
